# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.
#
# Author: Olivier Tilloy <olivier@fluendo.com>

from elisa.core.common import application
from elisa.core.media_uri import MediaUri
from elisa.core.components.model import Model
from elisa.core.application import PICTURES_CACHE
from elisa.core.utils.i18n import install_translation

from elisa.plugins.poblesec.link import Link
from elisa.plugins.poblesec.base.hierarchy import HierarchyController
from elisa.plugins.poblesec.base.list import GenericListViewMode
from elisa.plugins.poblesec.base.preview_list import MenuItemPreviewListController
from elisa.plugins.poblesec.base.coverflow import \
    ImageWithReflectionCoverflowController
from elisa.plugins.poblesec.base.grid import GridItemGridController
from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController

from elisa.plugins.youtube.models import YoutubeVideoModel, \
                                         YoutubeVideoListModel

from twisted.internet import defer

import os
import hashlib


_ = install_translation('youtube')


class YoutubeVideoFeedModel(Model):

    def __init__(self):
        super(Model, self).__init__()
        self.title = None
        self.uri = None


def youtube_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/youtube'
    link.label = _('Youtube')
    link.icon = 'elisa.plugins.youtube.logo'
    controller.model.append(link)
    return defer.succeed(None)


# TODO: factorize this code in the core of Elisa in a caching utilities module
def _thumbnail_file(thumbnail_uri):
    if not os.path.exists(PICTURES_CACHE):
        os.makedirs(PICTURES_CACHE, 0755)
    thumbnail = hashlib.md5(str(thumbnail_uri)).hexdigest() + '.jpg'
    return os.path.join(PICTURES_CACHE, thumbnail)


class YoutubeController(HierarchyController):

    start_uri = MediaUri('http://gdata.youtube.com/feeds/')

    def initialize(self, uri=start_uri):
        deferred = super(YoutubeController, self).initialize()
        self.uri = uri

        def add_feeds(self):
            feeds_root = 'http://gdata.youtube.com/feeds/api/standardfeeds/'
            feeds = [(_('Top rated'), 'top_rated'),
                     (_('Top favorites'), 'top_favorites'),
                     (_('Most viewed'), 'most_viewed'),
                     (_('Most popular'), 'most_popular'),
                     (_('Most recent'), 'most_recent')]
            feed_models = []
            for feed in feeds:
                feed_model = YoutubeVideoFeedModel()
                feed_model.title = feed[0]
                feed_model.uri = MediaUri(feeds_root + feed[1])
                feed_models.append(feed_model)
            self.model.extend(feed_models)
            return self

        def resource_loaded(resource):
            if isinstance(resource, YoutubeVideoListModel):
                self.model.extend(resource.videos)
            return self

        def load_resource(self):
            resource, get_deferred = application.resource_manager.get(self.uri)
            return get_deferred

        if uri == self.start_uri:
            # List of the feeds
            deferred.addCallback(add_feeds)
        else:
            # One particular feed
            deferred.addCallback(load_resource)
            deferred.addCallback(resource_loaded)

        return deferred

    def node_clicked(self, widget, item):
        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
        selected_item = self.nodes._widgets[widget_index]

        if selected_item != self._previous_clicked:
            selected_item.activate(previous=self._previous_clicked)

            self._previous_clicked = selected_item

            if isinstance(item, YoutubeVideoFeedModel):
                controllers = self.frontend.retrieve_controllers('/poblesec/browser')
                browser = controllers[0]
                dfr = browser.history.append_controller(self.path, item.title, uri=item.uri)

            elif isinstance(item, YoutubeVideoModel):
                def play_video(playable_model):
                    controllers = self.frontend.retrieve_controllers('/poblesec/video_player')
                    player = controllers[0]
                    player.player.play_model(playable_model)

                    controllers = self.frontend.retrieve_controllers('/poblesec')
                    main = controllers[0]
                    main.show_video_player()
                    self.stop_loading_animation()

                if item.playable_model:
                    # We already know the real URL of the playable video, no
                    # need to query the resource manager
                    play_video(item.playable_model)
                    dfr = defer.succeed(True)
                else:
                    playable_model, dfr = application.resource_manager.get(item.playable_uri, item)
                    dfr.addCallback(play_video)


class YoutubeViewMode(GenericListViewMode):

    """
    Implementation of the common view modes API.
    """

    def get_label(self, item):
        return defer.succeed(item.title)

    def get_default_image(self, item):
        if isinstance(item, YoutubeVideoFeedModel):
            resource = 'elisa.plugins.poblesec.video_folders'
        elif isinstance(item, YoutubeVideoModel):
            resource = 'elisa.plugins.poblesec.file_video'
        return resource

    def get_image(self, item, theme):
        if isinstance(item, YoutubeVideoModel):
            # Get a thumbnail for the video
            thumbnail_uri = item.thumbnails[0].references[0]
            try:
                thumbnail_file = item.thumbnail_file
            except AttributeError:
                item.thumbnail_file = _thumbnail_file(thumbnail_uri)
                thumbnail_file = item.thumbnail_file
            if os.path.exists(thumbnail_file):
                return defer.succeed(thumbnail_file)
            else:
                # Download the thumbnail first
                def got_thumbnail(data_model):
                    fd = file(thumbnail_file, 'wb')
                    fd.write(data_model.data)
                    fd.close()
                    return thumbnail_file

                data_model, thumbnail_deferred = \
                    application.resource_manager.get(thumbnail_uri)
                return thumbnail_deferred.addCallback(got_thumbnail)
        else:
            return None

    def get_preview_image(self, item, theme):
        if isinstance(item, YoutubeVideoModel):
            try:
                return item.thumbnail_file
            except AttributeError:
                return None
        else:
            return None


class YoutubePreviewListController(YoutubeController, MenuItemPreviewListController):
    view_mode = YoutubeViewMode


class YoutubeCoverflowController(YoutubeController, ImageWithReflectionCoverflowController):
    view_mode = YoutubeViewMode


class YoutubeGridController(YoutubeController, GridItemGridController):
    view_mode = YoutubeViewMode


class YoutubeListSwitcherController(ListSwitcherController):
    modes = [YoutubePreviewListController,
             YoutubeCoverflowController,
             YoutubeGridController]
    default_mode = YoutubePreviewListController
